require 'spec_helper'

describe ResourceNotFound do
  context 'as JSONError.base child-class' do
    describe 'NULL_TOKEN' do
      subject { described_class::NULL_TOKEN }
      it { is_expected.to eq('~') }
    end

    describe '.superclass' do
      subject { described_class.superclass }
      it { is_expected.to be(JSONError.base) }
    end

    describe '.base?' do
      subject { described_class.base? }
      it { is_expected.to be_falsey }
    end

    describe '.key' do
      context 'when no key was passed' do
        subject { described_class.key }
        it { is_expected.to eq(:resource_not_found) }
      end
    end

    describe '.properties' do
      subject { described_class.properties }
      let(:properties) do
        described_class.default_properties + %i(message description href details)
      end
      it { is_expected.to eq(properties) }
    end

    describe '.default_properties' do
      subject { described_class.default_properties }
      it { is_expected.to eq(%i(status key)) }
    end

    describe '.translated_properties' do
      subject { described_class.translated_properties }
      context 'when no args were passed' do
        it { is_expected.to eq(%i(message description href)) }
      end
    end

    describe '.custom_properties' do
      subject { described_class.custom_properties }

      context 'when no args were passed' do
        it { is_expected.to eq(%i(details)) }
      end
    end

    describe '.visible_properties' do
      subject { described_class.visible_properties }
      context 'when no args were passed' do
        it { is_expected.to eq(described_class.properties) }
      end
    end

    describe '.new' do
      context 'called without :key option' do
        subject { described_class.new }
        it { is_expected.to be_a(described_class) }
        describe '#key' do
          it { expect(subject.key).to eq(:resource_not_found) }
        end
      end

      context 'called with :key option' do
        subject { described_class.new(key: :other_key) }
        it { is_expected.to be_a(described_class) }
        describe '#key' do
          it { expect(subject.key).to eq(:resource_not_found) }
        end
      end
    end

    describe '#status' do
      context 'for default context' do
        subject { described_class.new.status }
        it { is_expected.to eq(404) }
      end

      context 'for context "other"' do
        subject { described_class.new(context: 'other').status }
        it { is_expected.to eq(404) }
      end
    end

    describe '#key' do
      subject { described_class.new.key }
      it { is_expected.to eq(:resource_not_found) }
    end

    describe '#message' do
      context 'for default context' do
        subject { described_class.new.message }
        it { is_expected.to eq('Resource %{name} (%{id}) was not found.') }
      end

      context 'for context "other"' do
        subject { described_class.new(context: 'other').message }
        it { is_expected.to eq('Resource not found.') }
      end
    end

    describe '#description' do
      context 'for default context' do
        subject { described_class.new.description }
        it { is_expected.to eq('Blah blah blah.') }
      end

      context 'for context "other"' do
        subject { described_class.new(context: 'other').description }
        it { is_expected.to eq('Bleh bleh bleh.') }
      end
    end

    describe '#href' do
      context 'for default context' do
        subject { described_class.new.href }
        it do
          is_expected.to eq('https://developers.api.pl/doc/errors/resource_not_found')
        end
      end

      context 'for context "other"' do
        subject { described_class.new(context: 'other').href }
        it do
          is_expected.to eq('https://developers.api.pl/doc/errors/resource_not_found?context=other')
        end
      end
    end

    describe '#to_json' do
      let(:parse) { -> (s) { JSON.parse(s) } }
      let(:error) { described_class.new }
      subject { error.to_json }

      describe '#keys' do
        let(:keys) { %w(status key message description href details) }
        it { expect(parse.call(subject).keys).to eq(keys) }
      end

      context 'when defined in locale' do
        it { is_expected.to be_a(String) }
        describe 'status property' do
          it { expect(parse.call(subject)['status']).to eq(error.status) }
        end
        describe 'key property' do
          it { expect(parse.call(subject)['key']).to eq(error.key.to_s) }
        end
        describe 'message property' do
          it { expect(parse.call(subject)['message']).to eq(error.message.to_s) }
        end
        describe 'description property' do
          it { expect(parse.call(subject)['description']).to eq(error.description.to_s) }
        end
        describe 'href property' do
          it { expect(parse.call(subject)['href']).to eq(error.href.to_s) }
        end
      end
    end
  end
end

describe MissingTranslations do
  context 'as JSONError.base child-class' do
    describe '.visible_properties' do
      subject { described_class.visible_properties }
      context 'when no args were passed' do
        it { is_expected.to eq(%i(status key message details)) }
      end
    end

    describe '#status' do
      subject { described_class.new.status }
      it { is_expected.to eq(404) }
    end

    describe '#key' do
      subject { described_class.new.key }
      it { is_expected.to eq(:missing_translations) }
    end

    describe '#message' do
      subject { described_class.new.message }
      it { is_expected.to be_nil }
    end

    describe '#description' do
      subject { described_class.new.description }
      it { is_expected.to be_nil }
    end

    describe '#href' do
      subject { described_class.new.href }
      it { is_expected.to be_nil }
    end

    describe '#to_json' do
      let(:parse) { -> (s) { JSON.parse(s) } }
      let(:details) { { 'detail' => 'value' } }
      let(:error) { described_class.new(details: details) }
      subject { error.to_json }

      describe '#keys' do
        let(:keys) { %w(status key message details) }
        it { expect(parse.call(subject).keys).to eq(keys) }
      end

      context 'when defined in locale' do
        it { is_expected.to be_a(String) }
        describe 'status property' do
          it { expect(parse.call(subject)['status']).to eq(error.status) }
        end
        describe 'key property' do
          it { expect(parse.call(subject)['key']).to eq(error.key.to_s) }
        end
        describe 'message property' do
          it { expect(parse.call(subject)['message']).to be_nil }
        end
        describe 'details property' do
          it { expect(parse.call(subject)['details']).to eq(error.details) }
        end
      end
    end
  end
end

describe ChildError do
  let(:parent_class) { MissingTranslations }
  context 'as MissingTranslations child-class' do
    describe '.visible_properties' do
      subject { described_class.visible_properties.sort }
      context 'when no args were passed' do
        it do
          is_expected.to eq(
            [*parent_class.properties, :additional_translated, :additional_custom].sort
          )
        end
      end
    end

    describe '.translated_properties' do
      subject { described_class.translated_properties.sort }
      context 'when no args were passed' do
        it do
          is_expected.to eq(
            [*parent_class.translated_properties, :additional_translated].sort
          )
        end
      end
    end

    describe '.custom_properties' do
      subject { described_class.custom_properties.sort }
      context 'when no args were passed' do
        it do
          is_expected.to eq(
            [*parent_class.custom_properties, :additional_custom].sort
          )
        end
      end
    end

    describe '#key' do
      subject { described_class.new.key }
      it { is_expected.to eq(:child_error) }
    end
  end
end

describe WithKeyAlias do
  let(:error_object) { described_class.new }
  let(:error_key) { :with_key_alias }

  describe '#key' do
    subject { error_object.key }
    it { is_expected.to eq(error_key) }
  end

  describe '#to_json' do
    let(:parse) { -> (s) { JSON.parse(s) } }
    subject { error_object.to_json }

    describe '#keys' do
      let(:keys) { %w(status error_code) }
      it { expect(parse.call(subject).keys).to eq(keys) }
    end

    describe 'status property' do
      it { expect(parse.call(subject)['status']).to eq(400) }
    end

    describe 'error_code property' do
      it { expect(parse.call(subject)['error_code']).to eq(error_key.to_s) }
    end
  end
end

describe WithKeyAliasChild do
  let(:error_object) { described_class.new }
  let(:error_key) { :with_key_alias_child }

  describe '#key' do
    subject { error_object.key }
    it { is_expected.to eq(error_key) }
  end

  describe '#to_json' do
    let(:parse) { -> (s) { JSON.parse(s) } }
    subject { error_object.to_json }

    describe '#keys' do
      let(:keys) { %w(status error_code) }
      it { expect(parse.call(subject).keys).to eq(keys) }
    end

    describe 'status property' do
      it { expect(parse.call(subject)['status']).to eq('429') }
    end

    describe 'error_code property' do
      it { expect(parse.call(subject)['error_code']).to eq(error_key.to_s) }
    end
  end
end
