require 'spec_helper'

describe JSONError::Base do
  describe 'DEFAULT_STATUS' do
    subject { described_class::DEFAULT_STATUS }
    it { is_expected.to eq(500) }
  end

  describe 'NULL_TOKEN' do
    subject { described_class::NULL_TOKEN }
    it { is_expected.to eq('~') }
  end

  describe '.superclass' do
    subject { described_class.superclass }
    it { is_expected.to be(StandardError) }
  end

  describe '.base?' do
    subject { described_class.base? }
    it { is_expected.to be_truthy }
  end

  describe '.key' do
    context 'when no key was passed' do
      subject { described_class.key }
      it { is_expected.to be_nil }
    end

    context 'when key were passed' do
      let(:error) { JSONError::Errors::DisallowedPropertySetting }
      subject { -> { described_class.key(:error) } }
      it { is_expected.to raise_error(error) }
    end
  end

  describe '.properties' do
    subject { described_class.properties }
    it { is_expected.to eq(described_class.default_properties) }
  end

  describe '.default_properties' do
    subject { described_class.default_properties }
    it { is_expected.to eq(%i(status key)) }
  end

  describe '.translated_properties' do
    subject { described_class.translated_properties(*args) }

    context 'when no args were passed' do
      let(:args) { [] }
      it { is_expected.to be_nil }
    end

    context 'when args were passed' do
      let(:args) { %i(message) }
      let(:error) { JSONError::Errors::DisallowedPropertySetting }
      it { expect { subject }.to raise_error(error) }
    end
  end

  describe '.custom_properties' do
    subject { described_class.custom_properties(*args) }

    context 'when no args were passed' do
      let(:args) { [] }
      it { is_expected.to be_nil }
    end

    context 'when args were passed' do
      let(:args) { %i(message) }
      let(:error) { JSONError::Errors::DisallowedPropertySetting }
      it { expect { subject }.to raise_error(error) }
    end
  end

  describe '.visible_properties' do
    subject { described_class.visible_properties(*args) }

    context 'when no args were passed' do
      let(:args) { [] }
      it { is_expected.to eq(described_class.properties) }
    end

    context 'when args were passed' do
      let(:args) { %i(message) }
      let(:error) { JSONError::Errors::DisallowedPropertySetting }
      it { expect { subject }.to raise_error(error) }
    end
  end

  describe '.new' do
    context 'called without :key option' do
      subject { -> { described_class.new } }
      it { is_expected.to raise_error(KeyError) }
    end

    context 'called with :key option' do
      subject { described_class.new(key: :resource_not_found) }
      it { is_expected.to be_a(described_class) }
    end
  end

  describe '#status' do
    let(:error) { described_class.new(key: :other_error) }
    subject { error.status }

    it { is_expected.to eq(500) }
  end

  describe '#key' do
    let(:error) { described_class.new(key: :resource_not_found) }
    subject { error.key }
    it { is_expected.to eq(:resource_not_found) }
  end

  describe '#to_json' do
    let(:parse) { -> (s) { JSON.parse(s) } }
    subject { error.to_json }

    context 'when not defined in locale' do
      let(:error) { described_class.new(key: :other_error) }
      it { is_expected.to be_a(String) }
      describe 'status property' do
        it { expect(parse.call(subject)['status']).to eq(500) }
      end
      describe 'key property' do
        it { expect(parse.call(subject)['key']).to eq(error.key.to_s) }
      end
    end

    context 'when defined in locale' do
      let(:error) { described_class.new(key: :resource_not_found) }
      it { is_expected.to be_a(String) }
      describe 'status property' do
        it { expect(parse.call(subject)['status']).to eq(error.status) }
      end
      describe 'key property' do
        it { expect(parse.call(subject)['key']).to eq(error.key.to_s) }
      end
    end
  end
end
